import pandas as pd
import os
import math

def calculate_grade(input_file_path):
    """
    When all course components are presented, calculate the course overall grade based on information provided.
    
    Parameters
    ----------
    input_file_path : str
        Path to read the saved .csv file as a string.
        Under the current working directory to avoid permission issue.
        The path should include the file name.
    
    Returns
    -------
    return_msg : double
        The course overall grade once all course components present when all grades are present.
        Otherwise return warning message
    
    Examples
    --------
    >>> calculate_grade('/DSCI524.csv')
    """
    cwd = os.getcwd()
    path = cwd + '/' + input_file_path
    course_info = pd.read_csv(path, index_col=0)
    print(course_info)

    error_msg_missing_value = 'Course component grades is missing for '
    error_msg_format = 'Course component grades with incorrect format (not 2 decimal places) for '
    grade_msg = 'Course garde is '
    percent_sign = '%'
    course_grade = 0
    return_msg = ''

    # Check if all course components have garde present
    for i, row in course_info.iterrows():
        comp = row['Components']
        grade = row['Grades (%)'] 
        weight = row['Weights (%)']
        # print(grade)
        # print(weight)

        if  math.isnan(grade):
            return_msg = error_msg_missing_value + comp
            return return_msg

        if len(str(grade).rsplit('.')[-1]) != 2:
            return_msg = error_msg_format + comp
            return return_msg

        course_grade += (grade * weight / 100)

    return_msg = grade_msg + str(round(course_grade, 2)) + percent_sign

    return return_msg


import pandas as pd
import os
import math

def predict_final(input_file, goal):
    """
    Calculate what grade the student needs to have for the final assignment in order to reach the goal for overall grade
    and return the scores needed for the final assignment.
    
    Parameters
    ----------
    input_file : str
        Path to read the saved .csv file as a string.
    goal : double
        The goal for your overall grade for this course.
    
    Returns
    -------
    final_score_needed : double
        The scores needed for the final assignment. 
        Otherwise return warning message.
    
    Examples
    --------
    >>> predict_final('/DSCI100.csv', 90.0)
    """
    # Set up input path
    cwd = os.getcwd()
    path = cwd + input_file
    course_info = pd.read_csv(path, index_col=0)
    
    # Check if all assignment weight sum up to 100%
    if course_info['Weights (%)'].sum() != 100:
        error_msg = "Your total assignment weights are not summing up to 100%! Please update all your assignments."
        final_score_needed = error_msg
        return final_score_needed
    
    # Check if final score (in the last row) is NA
    if not math.isnan(course_info['Grades (%)'].iloc[-1]):
        error_msg = "Your final assignment score is already updated. Please directly use calculate_grade()."
        final_score_needed = error_msg
        return final_score_needed
    
    # Check if there if more than one NA value in the 'Grades (%)' column
    if course_info['Grades (%)'].isnull().sum() > 1:
        error_msg = "Please update all your assignment scores except the final."
        final_score_needed = error_msg
        return final_score_needed
    
    # Calculate final_score_needed with given goal
    total_scores_except_final = (course_info['Grades (%)'].iloc[:-1] * course_info['Weights (%)'].iloc[:-1]).sum()
    final_score_needed = round((goal*100 - total_scores_except_final)/course_info['Weights (%)'].iloc[-1], 1)
    return final_score_needed
    
    
    


    

    
  


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['gradecalculatorpy']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.5.2,<2.0.0', 'pytest-cov>=4.0.0,<5.0.0']

setup_kwargs = {
    'name': 'gradecalculatorpy',
    'version': '0.0.1',
    'description': 'A python package which helps calculate grades for a course.',
    'long_description': "# gradecalculatorpy\n\n## Summary\n\nThis python package calculates grades for a course. The package allows users to customize their own course information with self-defined course component names, to update grades for different course components, and even understand how well the final exam needs to be to pass the course or achieve a target final grade.\n\n## Usage\n\n### Functions\n\nThis package contains the following functions:\n\n- `construct_course`: Allow users to input the information for one course component (for example, assignment name and corresponding weight) one by one. Saves the course information as a .csv file to the specified file path. (Note: any user self-defined course component name(s) can be accepted)\n    - **Note**: Since this function accepts user interaction input when calling the function, developer team had not yet found a way to pass the function unit test through *pytest*.<br>\n    The unit test scenarios covered in the `test_construct_course()` is based on the assumption that the input .csv file is successfully generated through a local testing by calling `construct_course('dsci524', '/')` for example\n\n- `update_grades`: Allow users to update course component grade(s) by loading a certain saved course .csv file. The function can then save the updated course information as a new .csv file to the specified file path.\n  \n- `predict_final`: Calculate how well the final exam has to be in order to pass the course or achieve a certain grade just before the final exam (for the scenario only when the final grade is missing), based on the provided other course component information.\n\n- `calculate_grade`: When all course components are presented, calculate the course overall grade based on information provided. \n\n## Suitability within Python Ecosystem\n\nThis course grade calculator is unique as it provides an interactive way for users to input the course component information and update information as needed. With the `predict_final` function available, users can understand how well the final exam has to be in order to pass the course or achieve a certain level of grade in this course, then adjust their final review plans based on our calculation, to meet the course expectation.\n\nThe package `predict_final` function does not take any users' previous study or course information into account to calculation of the desired final performance, only based on the current course component information inputted/updated. \n\nThere are other course grade calculators in the Python ecosystem. Some of the examples can be found [here](https://pypi.org/project/grade/) and [here](https://pypi.org/project/grade-tracker/). While other packages focus on auto-grading without user interaction and they do not provide similar functions like `predict_final` function in this package.\n\n## Installation\n\n```bash\n$ pip install gradecalculatorpy\n```\n\n## Usage\n\n## Acknowledging the use of OS\n\nSince Windows and Mac systems have different file path naming style and we run all our pytest successfully on Mac. Therefore, we highly recommend the TA grading this assignment using Mac.\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## Contributors\n\nMembers of Group 20 of DSCI524 at UBC: <br> Chen Lin, Edward Yukun Zhang, Shirley Zhang\n\n## License\n\n`gradecalculatorpy` was created by Chen Lin, Edward Yukun Zhang, Shirley Zhang. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`gradecalculatorpy` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n",
    'author': 'Chen Lin, Edward Zhang, Shirley Zhang',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/UBC-MDS/gradecalculatorpy',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import pandas as pd

def load_course(input_file):
    """Loads a .csv file containing information about the course ('course.csv') and saves it in a pandas dataframe.
    
    Parameters
    ----------
    input_file : str
        Path to the .csv file as a string.
    
    Returns
    -------
    course_df : DataFrame
        A pandas dataframe containing the information from the .csv file.
    
    Examples
    --------
    >>> load_course('DSCI100.csv')
    """
    
    df = pd.read_csv(input_file, index_col=0)
    return df
    
def save_course_csv(updated_course_df, output_file):
    """Saves the updated course dataframe as a .csv file. 
    
    Parameters
    ----------
    updated_course_df : DataFrame 
        A pandas dataframe containing the updated grades for the course.
    output_file: str 
        Path to the output .csv file as a string. 
    
    Returns
    -------
    None
    
    Examples
    --------
    >>> save_course_csv(DSCI100_df, 'DSCI100.csv')
    """
    
    updated_course_df.to_csv(output_file)
    return 

def update_assignment_grade(input_file, assignment, grade):
    """Updates the 'grade' column of 'course.csv' with the new grade inputted. Can add a grade if there was no grade for the assignment previously, or change the current grade of the assignment to a new one. Will call 'save_course_csv' to save the updated dataframe as 'course.csv'. 
    
    Parameters
    ----------
    input_file : str
        Path to the .csv file as a string. 
    assignment: str
        Name of the assignment to be updated as a string. 
    grade : double 
        Grade of the assignment as a double. 
    
    Returns
    -------
    None
    
    Examples
    --------
    >>> update_assignment_grade('DSCI100.csv', 'Assignment 1', 95.0)
    """
    
    df = load_course(input_file)          # load_course() has its own exceptions 
    row = df['Components'] == assignment    # find the row number with the assignment 
    df.loc[row, 'Grades (%)'] = round(grade, 2)     # update grade and round 
    save_course_csv(df, input_file)       # save course as .csv into original input_file location 
                                          # save_course_csv() has its own exceptions
        
    



import pandas as pd
import os

def construct_course(course_name, output_file_path):
    """
    Allow user to input course components' information (component name and coresponding weight) one by one.
    Course components could be the following for example:
    1. Assignment(s)
    2. Quiz(s)
    3. Lab(s)
    4. Midterm(s)
    5. Final(s)
    6. Any user self-defined course component name(s)

    Save the course information as a .csv file (named based on the 'course_name' paramater) 
    to the file path specified by 'output_file_path' paramater
    
    Parameters
    ----------
    course_name: str 
        The course name as a string. It will be used as the output file name 

    output_file_path: str 
        Path to the output .csv file (named based on the 'course_name' paramater) 
        under the current working directory as a string. 
        The purpose to save under the current working directory it to avoid permission issue.
    
    Returns
    -------
    None

    Examples
    --------
    >>> construct_course('dsci524', '/')
    """
    
    course_total_weight = 0
    course_df = pd.DataFrame(columns = ["Components", "Weights (%)", "Grades (%)"])
    index = 0

    while course_total_weight < 100:
        curr_component = []

        curr_component_name = input(f"What is name of {course_name} component #{index+1}? ")

        # Check if the weight input can be parsed as integer
        curr_component_weight_input = input(f"What is weight(%) of {course_name} component #{index+1}? ")
        while not curr_component_weight_input.isdigit():
            curr_component_weight_input = input(f"Input weight(%) of {course_name} component #{index+1} as integer? ")

        curr_component_weight = int(curr_component_weight_input)

        # Check if the total weight not over 100%
        while course_total_weight + curr_component_weight > 100:
            curr_component_weight = int(input(f"Double check the weight of {course_name} component #{index+1}, ensure total weight not over 100%? "))

        curr_component.append(curr_component_name)
        curr_component.append(curr_component_weight)
        curr_component.append(float('nan'))

        course_df.loc[index] = curr_component

        course_total_weight += curr_component_weight

        index += 1

    # save course construction dataframe as csv file
    cwd = os.getcwd()
    path = cwd + output_file_path
    filepath = os.path.join(path, course_name + ".csv")
    course_df.to_csv(filepath)

    print(course_df)